@@ -95,7 +95,7 @@ public abstract class Expression extends Procedure
       {
 	OutPort port = new OutPort((PrintWriter) out);
 	print(port);
-	port.close();
+        port.closeThis();
       }
     else
       {
@@ -110,7 +110,7 @@ public abstract class Expression extends Procedure
 
   /**
    * Print line and column number if specified.
-   * This is a helper routineintended for use by print(OutPort).
+   * This is a helper routine intended for use by print(OutPort).
    */
   public void printLineColumn(OutPort out)
   {
@@ -293,6 +293,13 @@ public abstract class Expression extends Procedure
     this.filename = filename;
   }
 
+    public final void setLine(int startLine, int startColumn,
+                              int endLine, int endColumn)
+    {
+        position = SourceMapper.simpleEncode(startLine, startColumn,
+                                             endLine, endColumn);
+    }
+
   public final void setLine (int lineno, int colno)
   {
     position = SourceMapper.simpleEncode(lineno, colno);
