@@ -46,22 +46,19 @@ public class FindCapturedVars extends ExpExpVisitor<Void>
       {
 	Declaration decl
 	  = Declaration.followAliases(((ReferenceExp) exp.func).binding);
-        if (decl != null && ! decl.isPublic()) {
-            Expression value = decl.getValue();
-            if (decl.context instanceof ModuleExp
-                && ! decl.getFlag(Declaration.NONSTATIC_SPECIFIED)) {
-                if (value instanceof LambdaExp) {
-                    LambdaExp lexp = (LambdaExp) value;
-                    if (! lexp.getNeedsClosureEnv()
-                        && ! lexp.getFlag(LambdaExp.HAS_NONTRIVIAL_PATTERN))
-                        skipFunc = true;
-                }
-            } else if (value instanceof LambdaExp) {
-                Declaration ndecl = ((LambdaExp) value).nameDecl;
-                if (ndecl != null)
-                    capture(ndecl, null);
-            }
-        }
+	if (decl != null && decl.context instanceof ModuleExp
+            && ! decl.isPublic()
+            && ! decl.getFlag(Declaration.NONSTATIC_SPECIFIED))
+	  {
+	    Expression value = decl.getValue();
+	    if (value instanceof LambdaExp)
+	      {
+		LambdaExp lexp = (LambdaExp) value;
+		if (! lexp.getNeedsClosureEnv()
+                    && ! lexp.getFlag(LambdaExp.HAS_NONTRIVIAL_PATTERN))
+                  skipFunc = true;
+	      }
+	  }
       }
     // Similar hack for constructor calls, but here we want to
     // avoid visiting the type argument.
@@ -465,9 +462,6 @@ public class FindCapturedVars extends ExpExpVisitor<Void>
 		outer = heapLambda.outerLambda();
 	      }
 	  }
-        while (declLambda.getInlineOnly() && declLambda.getCaller() != null) {
-            declLambda = declLambda.getCaller();
-        }
         declLambda.capture(decl);
       }
   }
