@@ -62,7 +62,7 @@ public class LambdaExp extends ScopeExp {
         if (decl.isSimple()) {
             if (capturedVars == null
                 && ! decl.isStatic()
-                && ! (isClassGenerated() && decl.context == this)) {
+                && ! isClassGenerated()) {
                 heapFrame = new gnu.bytecode.Variable("$heapFrame");
             }
             decl.setSimple(false);
@@ -90,15 +90,6 @@ public class LambdaExp extends ScopeExp {
      */
     Variable heapFrame;
 
-    public Variable getHeapFrame() {
-        for (LambdaExp lexp = this; ; lexp = lexp.inlineHome) {
-            if (lexp.heapFrame != null
-                || lexp instanceof ModuleExp
-                || ! lexp.getInlineOnly())
-                return lexp.heapFrame;
-        }
-    }
-
     public LambdaExp firstChild;
     public LambdaExp nextSibling;
 
@@ -283,11 +274,6 @@ public class LambdaExp extends ScopeExp {
         return (flags & CLASS_METHOD) != 0;
     }
 
-    public boolean useClassForFrame() {
-        return this instanceof ClassExp
-            || (this instanceof ModuleExp && this.heapFrame == null);
-    }
-
     public final void setClassMethod(boolean isMethod) {
         if (isMethod) flags |= CLASS_METHOD;
         else flags &= ~CLASS_METHOD;
@@ -545,7 +531,7 @@ public class LambdaExp extends ScopeExp {
                 parent = parent.outerLambda();
             if (isClassMethod() && ! "*init*".equals(getName()))
                 closureEnv = declareThis(compiledType);
-            else if (parent.getHeapFrame() == null && ! parent.getNeedsClosureEnv()
+            else if (parent.heapFrame == null && ! parent.getNeedsStaticLink()
                      && ! (parent instanceof ModuleExp))
                 closureEnv = null;
             else if (! isClassGenerated() && ! getInlineOnly()) {
@@ -572,8 +558,10 @@ public class LambdaExp extends ScopeExp {
             } else {
                 if (inlineHome != null)
                     inlineHome.declareClosureEnv();
-                Variable hf = parent.getHeapFrame();
-                closureEnv = hf != null ? hf : parent.closureEnv;
+                closureEnv =
+                    parent.heapFrame != null && parent == outerLambda()
+                    ? parent.heapFrame
+                    : parent.closureEnv;
             }
         }
         return closureEnv;
@@ -599,12 +587,9 @@ public class LambdaExp extends ScopeExp {
             curLambda = curLambda.getCaller();
 
         gnu.bytecode.CodeAttr code = comp.getCode();
-        if (this == curLambda) {
-            Variable hf = curLambda.getHeapFrame();
-            if (hf != null) {
-                code.emitLoad(hf);
-                return;
-            }
+        if (curLambda.heapFrame != null && this == curLambda) {
+            code.emitLoad(curLambda.heapFrame);
+            return;
         }
         ClassType curType;
         if (curLambda.closureEnv != null) {
@@ -739,9 +724,12 @@ public class LambdaExp extends ScopeExp {
         else {
             // Similar to getOwningLambda(), but we can't add apply methods
             // to a ClassExp - at least not unless it extends ModuleBody.
-            do {
+            for (;;) {
                 owner = owner.outerLambda();
-            } while (! (owner instanceof ModuleExp) && owner.heapFrame == null);
+                if (owner instanceof ModuleExp
+                    || owner.heapFrame != null)
+                    break;
+            }
             ClassType frameType = owner.getHeapFrameType();
             if (! (frameType.getSuperclass().isSubtype(Compilation.typeCompiledProc)))
                 owner = comp.getModule();
@@ -863,10 +851,9 @@ public class LambdaExp extends ScopeExp {
                 else {
                     LambdaExp parent = comp.curLambda;
                     while (parent.getInlineOnly() && parent.heapFrame == null)
-                        parent = parent.outerLambdaOrCaller();
-                    Variable hf = parent.getHeapFrame();
+                        parent = parent.outerLambda();
                     Variable frame
-                        = hf != null ? hf
+                        = parent.heapFrame != null ? parent.heapFrame
                         : parent.closureEnv;
                     code.emitLoad(frame);
                     code.emitGetField(field);
@@ -877,16 +864,16 @@ public class LambdaExp extends ScopeExp {
     }
 
     public ClassType getHeapFrameType() {
-        if (useClassForFrame())
+        if (isClassGenerated())
             return (ClassType) getType();
         else
-            return (ClassType) getHeapFrame().getType();
+            return (ClassType) heapFrame.getType();
     }
 
 
     public LambdaExp getOwningLambda() {
-        LambdaExp exp = outerLambdaOrCaller();
-        for (;; exp = exp.outerLambdaOrCaller()) {
+        ScopeExp exp = getOuter();
+        for (;; exp = exp.getOuter()) {
             if (exp == null)
                 return null;
             if (exp instanceof ModuleExp
@@ -1257,7 +1244,7 @@ public class LambdaExp extends ScopeExp {
 
         Declaration decl = firstDecl();
         for (;;) {
-            if (! getInlineOnly()
+            if (! getInlineOnly() && ! inlinedInCheckMethod()
                 && getCallConvention() >= Compilation.CALL_WITH_CONSUMER
                 && decl == null) {
                 Variable var =
@@ -1318,14 +1305,9 @@ public class LambdaExp extends ScopeExp {
             closureEnvType = outer.getCompiledClassType(comp);
         else {
             LambdaExp owner = outer;
-            while (owner.heapFrame == null
-                   && ! (owner instanceof ModuleExp)) {
-                owner = owner.outerLambdaOrCaller();
-            }
-            if (owner.heapFrame != null)
-                closureEnvType = (ClassType) owner.heapFrame.getType();
-            else
-                closureEnvType = owner.compiledType;
+            while (owner.heapFrame == null)
+                owner = owner.outerLambda();
+            closureEnvType = (ClassType) owner.heapFrame.getType();
         }
         addMethodFor(comp, closureEnvType);
     }
@@ -1356,9 +1338,10 @@ public class LambdaExp extends ScopeExp {
                     if (isClassGenerated())
                         parentFrameType = (ClassType) getType();
                     else {
-                        Variable frame = this.getHeapFrame();
-                        parentFrameType = (ClassType)
-                            (frame != null ? frame : this.closureEnv).getType();
+                        Variable parentFrame = this.heapFrame != null
+                            ? this.heapFrame
+                            : this.closureEnv;
+                        parentFrameType = (ClassType) parentFrame.getType();
                     }
                     cl.closureEnvField = cl.staticLinkField
                         = cl.instanceType.setOuterLink(parentFrameType);
@@ -1370,7 +1353,7 @@ public class LambdaExp extends ScopeExp {
     public void allocFrame(Compilation comp) {
         if (heapFrame != null)  {
             ClassType frameType;
-            if (useClassForFrame())
+            if (isClassGenerated())
                 frameType = getCompiledClassType(comp);
             else  {
                 frameType = new ClassType(comp.generateClassName("frame"));
@@ -1429,16 +1412,12 @@ public class LambdaExp extends ScopeExp {
                 decl.makeField(frameType, comp, null);
             }
         }
-        Variable hframe = getHeapFrame();
-        if (hframe != null && ! comp.usingCPStyle()
-            && ! getInlineOnly()) {
-            ClassType frameType = (ClassType) hframe.getType();
-            if (closureEnv != null && ! (this instanceof ModuleExp)
-                && ! getInlineOnly()) {
+        if (heapFrame != null && ! comp.usingCPStyle()) {
+            ClassType frameType = (ClassType) heapFrame.getType();
+            if (closureEnv != null && ! (this instanceof ModuleExp))
                 staticLinkField = frameType.addField("staticLink",
                                                      closureEnv.getType());
-            }
-            if (! useClassForFrame()) {
+            if (! isClassGenerated()) {
                 frameType.setEnclosingMember(comp.method);
                 code.emitNew(frameType);
                 code.emitDup(frameType);
@@ -1450,11 +1429,9 @@ public class LambdaExp extends ScopeExp {
                     code.emitLoad(closureEnv);
                     code.emitPutField(staticLinkField);
                 }
-                hframe.allocateLocal(code);
-                code.emitStore(hframe);
-                if (this.heapFrame != null && this.heapFrame.getScope() == null) {
-                    code.pushAutoPoppableScope().addVariable(hframe);
-                }
+                heapFrame.allocateLocal(code);
+                code.emitStore(heapFrame);
+                code.pushAutoPoppableScope().addVariable(heapFrame);
             }
         }
 
@@ -1478,9 +1455,7 @@ public class LambdaExp extends ScopeExp {
                 Type stackType = paramType;
                 // If the parameter is captured by an inferior lambda,
                 // then the incoming parameter needs to be copied into its
-                // slot in the heapFrame.  Thus we emit an aaload instruction.
-                // Unfortunately, it expects the new value *last*,
-                // so first push the heapFrame array and the array index.
+                // slot in the heapFrame.
                 if (!param.isSimple())
                     param.loadOwningObject(null, comp);
                 // This part of the code pushes the incoming argument.
