@@ -725,6 +725,7 @@ public abstract class Language
   public static final int PARSE_EXPLICIT = 64;
   public static final int PARSE_INTERACTIVE_MODULE = 128;
   public static final int PARSE_EMIT_MAIN = 256;
+  public static final int PARSE_FOR_LINT = 512;
 
   public static boolean requirePedantic;
 
@@ -1069,26 +1070,21 @@ public abstract class Language
         return null;
     }
 
-  public static/* for now */ Type unionType (Type t1, Type t2)
-  {
-    if (t1 == Type.neverReturnsType)
-      return t2;
-    if (t2 == Type.neverReturnsType)
-      return t1;
-    if (t1 == t2)
-      return t1;
-    if (t1 == Type.toStringType)
-      t1 = Type.javalangStringType;
-    if (t2 == Type.toStringType)
-      t2 = Type.javalangStringType;
-    if (t1.isVoid() || t2.isVoid())
-        return Type.objectType;
-    if (t1.isSubtype(t2))
-        return t2;
-    if (t2.isSubtype(t1))
-        return t1;
-    return Type.objectType;
-  }
+    public static/* for now */ Type unionType (Type t1, Type t2) {
+        if (t1 == Type.neverReturnsType)
+            return t2;
+        if (t2 == Type.neverReturnsType)
+            return t1;
+        if (t1 == t2)
+            return t1;
+        if (t1 == Type.toStringType)
+            t1 = Type.javalangStringType;
+        if (t2 == Type.toStringType)
+            t2 = Type.javalangStringType;
+        if (t1.isVoid() || t2.isVoid())
+            return Type.objectType;
+        return Type.lowestCommonSharedType(t1, t2);
+    }
 
     public Declaration declFromField(ModuleExp mod, Object fvalue, Field fld) {
         String fname = fld.getName();
