@@ -1,5 +1,6 @@
 package gnu.kawa.functions;
 
+import gnu.bytecode.ClassType;
 import gnu.bytecode.Type;
 import gnu.expr.*;
 import gnu.mapping.Procedure;
@@ -9,10 +10,13 @@ public class CompileProcess {
     /** Type-check and optimize RunProcess (i.e. 'run-process'). */
     public static Expression validateApplyRunProcess
         (ApplyExp exp, InlineCalls visitor, Type required, Procedure proc) {
+        if (exp.firstSpliceArg >= 0)
+            return exp;
         exp.visitArgs(visitor);
         Expression[] args = exp.getArgs();
         int nargs = args.length;
         int inArg = -1;
+        boolean returnBlob = true;
         for (int i = 0;  i < nargs; i++) {
             Expression arg = args[i];
             Keyword key = arg.checkLiteralKeyword();
@@ -20,6 +24,8 @@ public class CompileProcess {
                 String name = key.getName();
                 if ("in".equals(name))
                     inArg = i+1;
+                if (name.startsWith("out"))
+                    returnBlob = false;
                 i++;
             } else if (inArg < 0 && i+1 < nargs)
                 inArg = i;
@@ -42,6 +48,12 @@ public class CompileProcess {
                 }
             }
         }
+        /* FUTURE (causes type-checking problems for now)
+        // If returnBlob, use a new LangObjType with custom compare method.
+        Type t = ClassType.make(returnBlob ? "gnu.kawa.functions.LProcess"
+                                : "java.lang.Process");
+        exp.setType(t);
+        */
         return exp;
     }
 }
