@@ -368,7 +368,7 @@ public class OccurrenceType extends Type
   }
 
     /* #ifndef JAVA8 */
-    public String encodeType(Language language) { return null; }
+    // public String encodeType(Language language) { return null; }
     /* #endif */
 
   public static final ClassType typeOccurrenceType
