@@ -1,30 +1,30 @@
 package gnu.lists;
 import java.io.*;
 import gnu.text.SourceLocator;
+import gnu.text.SourceMapper;
 
-/** A <code>Pair</code> with the file name and position it was read from. */
+/** A <code>Pair</code> with the file name and position it was read from.
+ * Note the position (start/end range) is actually that of the car part. */
 
 public class PairWithPosition extends ImmutablePair
   implements gnu.text.SourceLocator
 {
-  String filename;
-  /** An encoding of {@code (lineNumber << 12) + columnNumber}.
-   * Note if columnNumber is unspecified (0), then position is lineNumber. */
-  int position;
+    String filename; // Future: union(String,SourceMapper)
+    long position; // See SourceMapper#simpleEncode
 
   public final void setFile (String filename)
   {
     this.filename = filename;
   }
 
-  public final void setLine (int lineno, int colno)
-  {
-    if (lineno < 0)
-      lineno = 0;
-    if (colno < 0)
-      colno = 0;
-    position = (lineno << 12) + colno;
-  }
+    public final void setLine(int lineno, int colno) {
+        position = SourceMapper.simpleEncode(lineno, colno);
+    }
+
+    public final void setEndLine(int endline, int endcolumn) {
+        position = SourceMapper.simpleEncode(getStartLine(), getStartColumn(),
+                                             endline, endcolumn);
+    }
 
   public final void setLine (int lineno)
   {
@@ -46,19 +46,29 @@ public class PairWithPosition extends ImmutablePair
     return filename;
   }
 
-  /** Get the line number of (the start of) this Expression.
-    * The "first" line is line 1; unknown is -1. */
-  public final int getLineNumber()
-  {
-    int line = position >> 12;
-    return line == 0 ? -1 : line;
-  }
+    public final int getLineNumber() {
+        return SourceMapper.simpleStartLine(position);
+    }
 
-  public final int getColumnNumber()
-  {
-    int column = position & ((1 << 12) - 1);
-    return column == 0 ? -1 : column;
-  }
+    public final int getColumnNumber() {
+        return SourceMapper.simpleStartColumn(position);
+    }
+
+    public final int getStartLine() {
+        return SourceMapper.simpleStartLine(position);
+    }
+
+    public final int getStartColumn() {
+        return SourceMapper.simpleStartColumn(position);
+    }
+
+    public final int getEndLine() {
+        return SourceMapper.simpleEndLine(position);
+    }
+
+    public final int getEndColumn() {
+        return SourceMapper.simpleEndColumn(position);
+    }
 
   public boolean isStableSourceLocation() { return true; }
 
@@ -72,7 +82,8 @@ public class PairWithPosition extends ImmutablePair
   {
     super (car, cdr);
     filename = where.getFileName();
-    setLine(where.getLineNumber(), where.getColumnNumber());
+    this.position = SourceMapper.simpleEncode(where.getStartLine(), where.getStartColumn(),
+                              where.getEndLine(), where.getEndColumn());
   }
 
   public PairWithPosition (Object car, Object cdr)
@@ -90,7 +101,7 @@ public class PairWithPosition extends ImmutablePair
   }
 
   public static PairWithPosition make(Object car, Object cdr,
-				      String filename, int position)
+				      String filename, long position)
   {
     PairWithPosition pair = new PairWithPosition(car, cdr);
     pair.filename = filename;
@@ -100,7 +111,7 @@ public class PairWithPosition extends ImmutablePair
 
     /** Should only be used when initializing a PairWithPosition instance. */
     public void init(Object car, Object cdr,
-                     String filename, int position) {
+                     String filename, long position) {
         this.car = car;
         this.cdr = cdr;
         this.filename = filename;
@@ -110,14 +121,14 @@ public class PairWithPosition extends ImmutablePair
   /**
    * @serialData Write the car followed by the cdr,
    *   followed by filename (as an {@code Object}, so it can be shared),
-   *   followed by position {@code (line|(column<<20))}.
+   *   followed by position (see SourceMapper#simpleEncode}.
    */
   public void writeExternal(ObjectOutput out) throws IOException
   {
     out.writeObject(car);
     out.writeObject(cdr);
     out.writeObject(filename);
-    out.writeInt(position);
+    out.writeLong(position);
   }
 
     public void readExternal(ObjectInput in)
@@ -125,7 +136,7 @@ public class PairWithPosition extends ImmutablePair
         Object car = in.readObject();
         Object cdr = in.readObject();
         String filename = (String) in.readObject();
-        int position = in.readInt();
+        long position = in.readLong();
         init(car, cdr, filename, position);
     }
 }
