@@ -4,96 +4,16 @@ import org.domterm.*;
 import org.domterm.util.DomTermErrorWriter;
 import org.domterm.util.StyleSheets;
 import org.domterm.util.Utf8WriterOutputStream;
-import org.domterm.DomHttpServer;
 import gnu.expr.*;
 import gnu.mapping.*;
 import gnu.kawa.io.*;
-import java.awt.Desktop;
 import java.io.*;
-import java.net.UnknownHostException;
-import java.net.URI;
 
 /** An implementation of DomTerm's Backend that runs a Kawa REPL. */
 
 public class DomTermBackend extends Backend implements Runnable {
 
     Thread thread;
-
-    public static class Server extends DomHttpServer {
-        private static Server instance = null;
-        Backend pendingBackend;
-
-        public Server(int port) throws UnknownHostException, IOException {
-            super(port, new String[0]);
-        }
-        @Override
-        protected Backend createBackend() {
-             if (pendingBackend != null) {
-                 Backend b = pendingBackend;
-                 pendingBackend = null;
-                 return b;
-             }
-            return new DomTermBackend();
-        }
-        public synchronized static Server getInstance() throws IOException {
-            if (instance == null) {
-                try {
-                    instance = new Server(0);
-                    instance.pendingBackend = new DomTermBackend();
-                } catch (UnknownHostException ex) {
-                    throw new RuntimeException(ex);
-                }
-                instance.start();
-            }
-            return instance;
-        }
-        public static int getInstancePort() throws IOException {
-            return getInstance().getPort();
-        }
-    }
-
-    public static String startDomTermConsole(String command) throws Throwable {
-        int htport = 0;
-        if (command.startsWith("serve=")) {
-            String portArg = command.substring(6);
-            try {
-                htport = Integer.parseInt(portArg);
-            } catch (NumberFormatException ex) {
-                return "bad port specifier in -w"+command+" option";
-            }
-        }
-        if ("google-chrome".equals(command)
-            || "chrome".equals(command))
-            command = "browser="+DomHttpServer.chromeCommand()+" --app=%U";
-        else if ("browser=google-chrome".equals(command)
-            || "browser=chrome".equals(command))
-            command = "browser="+DomHttpServer.chromeCommand()+" %U";
-        if ("firefox".equals(command)
-            || "browser=firefox".equals(command))
-            command = "browser="+DomHttpServer.firefoxCommand()+" %U";
-        boolean exitOnClose = ! command.startsWith("serve");
-        int port = Server.getInstancePort();
-        DomHttpServer.setExitOnClose(exitOnClose);
-        String url =  "http://localhost:"+port+"/domterm/#ajax";
-        if (command.equals("browser")) {
-            if (! Desktop.isDesktopSupported())
-                return "using default desktop browser not supported";
-            Desktop.getDesktop().browse(new URI(url));
-            return null;
-        } else if (command.startsWith("browser=")) {
-            String cmd = command.substring(8);
-            if (cmd.indexOf('%') < 0)
-                cmd = cmd  + " %U";
-            cmd = cmd.replace("%U", url).replace("%W", Integer.toString(port));
-            Runtime.getRuntime().exec(cmd);
-            return null;
-        }
-        else if (command.startsWith("serve")) {
-            return null;
-        }
-        return "unrecognized -w subcommand '"+command+"'";
-    }
-
     Language language;
     QueueReader inIn;
     Appendable inOut;
@@ -185,7 +105,7 @@ public class DomTermBackend extends Backend implements Runnable {
         */
         try {
             sendInputMode(usingJLine ? 'c' : 'p');
-            setAutomaticNewline(true);
+            termWriter.write("\033[20;1h"); // treat "\n" output as "\r\n"
             termWriter.write("\033]0;Kawa\007");
         } catch (Throwable ex) { ex.printStackTrace(); }
         if (this.nrows >= 0)
