@@ -26,6 +26,10 @@ public class SyntaxForms {
                                                   scope);
         if (datum instanceof Pair)
             return new PairSyntaxForm((Pair) datum, scope);
+        /* FUTURE
+        if (datum instanceof Symbol)
+            return scope.getAliasSymbol((Symbol) datum);
+        */
         return new SimpleSyntaxForm(datum, scope);
     }
 
@@ -151,6 +155,12 @@ public class SyntaxForms {
     return tr.rewrite(x);
   }
 
+  public static Expression rewriteCar (Object x)
+  {
+    Translator tr = (Translator) Compilation.getCurrent();
+    return tr.rewrite_car((Pair) x, false);
+  }
+
   public static Expression rewriteBody (Object x)
   {
     Translator tr = (Translator) Compilation.getCurrent();
