@@ -780,7 +780,7 @@ public class Translator extends Compilation
 
     /** Re-write a Scheme expression in S-expression format into internal form.
      * @param mode either 'N' (normal), 'F' (function application context),
-     *  or 'M' (macro-checking).
+     *  'M' (macro-checking) or 'Q' (colon-form in quote).
      */
     public Expression rewrite(Object exp, char mode) {
         if (exp instanceof SyntaxForm) {
@@ -794,7 +794,7 @@ public class Translator extends Compilation
             }
         }
         boolean function = mode != 'N';
-        if (exp instanceof Pair) {
+        if (exp instanceof Pair && mode != 'Q') {
             Expression e = rewrite_pair((Pair) exp, function);
             setLineOf(e);
             return e;
@@ -868,8 +868,13 @@ public class Translator extends Compilation
                                                       ctype, language);
                         if (methods.length == 0)
                             continue;
-                    } else if (decl != null && ! dname.equals(part.getName())) {
-                        continue;
+                    } else {
+                        gnu.expr.SourceName snameAnn =
+                            part.getAnnotation(gnu.expr.SourceName.class);
+                        String pname = snameAnn == null ? part.getName()
+                            : snameAnn.name();
+                        if (! dname.equals(pname))
+                            continue;
                     }
                     Expression part1;
                     // FIXME We're throwing away 'part', which is wasteful.
@@ -988,8 +993,9 @@ public class Translator extends Compilation
             }
 
             if (decl == null && function
-                && nameToLookup==LispLanguage.lookup_sym)
+                && nameToLookup==LispLanguage.lookup_sym) {
                 decl = getNamedPartDecl;
+            }
             int scanNesting = decl == null ? 0
                 : Declaration.followAliases(decl).getScanNesting();
             if (scanNesting > 0) {
@@ -1063,9 +1069,11 @@ public class Translator extends Compilation
         }
         if (namespaceName == LispLanguage.entityNamespace.getName()) {
             Object val = lookupStandardEntity(local);
-            if (val != null)
-                return makeQuoteExp(val);
-            tr.error('e', "unknown entity name "+local);
+            if (val == null) {
+                tr.error('e', "unknown entity name "+local);
+                val = "{"+namespace.getPrefix()+":"+local+"}";
+            }
+            return makeQuoteExp(val);
         }
 
         char ch0 = name.charAt(0);
@@ -1291,15 +1299,9 @@ public class Translator extends Compilation
         } catch (Exception ex) {
         }
         if (name.startsWith("array")) {
-            int nlen = name.length();
-            if (nlen == 5)
-                return makeQuoteExp(GenArrayType.generalInstance);
-            try {
-                rank = Integer.parseInt(name.substring(5));
-                if (rank >= 0)
-                    return makeQuoteExp(new GenArrayType(rank, Type.objectType));
-            } catch (Throwable ex) {
-            }
+            Type atype = LispLanguage.decodeArrayType(name);
+            if (atype != null)
+                return makeQuoteExp(atype);
         }
 
         return null;
@@ -1447,8 +1449,7 @@ public class Translator extends Compilation
 	       error('e', "unknown type name '"
 		     + ((ReferenceExp) texp).getName() + '\'');
 	     else
-	       error('e',
-		 "invalid type spec");
+               error('e', "invalid type spec");
              type = Type.errorType;
 	   }
         if (decl != null)
