@@ -1,20 +1,27 @@
-(module-export string? make-string $make$string$ string-length
+(module-export istring? string? make-string string-length
                string-ref string-set!
                char=? char<? char>? char<=? char>=?
                char-ci=? char-ci<? char-ci>? char-ci<=? char-ci>=?
                string=? string<? string>? string<=? string>=?
 	       string-ci=? string-ci<? string-ci>? string-ci<=? string-ci>=?
-               substring string->list list->string string-copy string-copy!
+               string-copy string-copy!
+               string-fold string-fold-right
+               string->list string-null?
                string-fill! string-upcase! string-downcase!
                string-capitalize string-capitalize!
-               string-append string-append! string-replace!
-               string-map string-for-each srfi-13-string-for-each)
+               string-append! string-replace!
+               string-for-each string-for-each-index srfi-13-string-for-each
+               string-pad string-pad-right
+               string-repeat string-tabulate
+               string-unfold string-unfold-right
+               string->utf8 string->utf16 string->utf16be string->utf16le
+               xsubstring)
 
-(require <kawa.lib.prim_syntax>)
+(require <kawa.lib.prim_imports>)
 (require <kawa.lib.std_syntax>)
 (require <kawa.lib.syntax>)
 (require <kawa.lib.lists>)
-(require <kawa.lib.characters>)
+(require kawa.lib.strings_syntax)
 (import (kawa lib kawa string-cursors))
 
 (define-syntax define-compare
@@ -31,41 +38,31 @@
                       (and (OP (COMP2 prev next) 0)
                            (loop (+ i 1) next)))))))))))
 
-(define-syntax with-start-end
-  (syntax-rules ()
-    ((_ str (start end) (cstart cend) . body)
-     (let* ((cstart (java.lang.Character:offsetByCodePoints str 0 start))
-            (cend (cond ((= end -1) (str:length))
-                        ((< end start)
-                         (primitive-throw
-                          (java.lang.StringIndexOutOfBoundsException)))
-                        (else
-                         (java.lang.Character:offsetByCodePoints
-                          str cstart (- end start))))))
-       . body))))
-
 (define (string? x) :: <boolean>
   (instance? x <string>))
 
-(define (make-string n ::int #!optional (ch ::character #\Space))
+(define (istring? x) ::boolean
+  (instance? x gnu.lists.IString))
+
+(define (string-null? str::string)
+  (= 0 (str:length)))
+
+(define (make-string #!optional (n ::int 0) (ch ::character #\Space))
   ::gnu.lists.FString
   (make <gnu.lists.FString> n (as int ch)))
 
-(define ($make$string$ #!rest args ::object[]) :: <string>
-  (let* ((n :: <int> args:length)
-	 (str (gnu.lists.FString:alloc n)))
-    (do ((i :: <int> 0 (+ i 1)))
-	((>= i n) str)
-	(str:appendCharacter
-         ((as gnu.text.Char (args i)):intValue)))))
+(define (string-repeat ch count::int) ::string
+  (let ((result (gnu.lists.FString:alloc count)))
+    (do ((i ::int 0 (+ i 1)))
+        ((>= i count)
+         (gnu.lists.IString result))
+      (result:append ch))))
 
 (define (string-length str ::string) ::int
   (gnu.lists.Strings:sizeInCodePoints str))
 
 (define (string-ref (str ::java.lang.CharSequence) (k ::int)) ::character
- (as character (java.lang.Character:codePointAt
-                str
-                (java.lang.Character:offsetByCodePoints str 0 k))))
+ (as character (gnu.lists.Strings:indexByCodePoints str k)))
 
 (define (string-set! str::abstract-string k::int char::character)
   ::void
@@ -82,16 +79,11 @@
 (define-compare string<=? string <= %string-compare2)
 (define-compare string>=? string >= %string-compare2)
 
-(define (substring str::<string> start::<int> end::<int>)
-  :: <string>
-  (with-start-end str (start end) (istart iend)
-                  (gnu.lists.FString str istart (- iend istart))))
-
 (define (string->list (str ::string)
-                      #!optional (start ::int 0) (end ::int -1))
+                      #!optional (start ::int 0) (end ::int 0 supplied-end))
   ::list
   (with-start-end 
-   str (start end) (cstart cend)
+   str (start end supplied-end) (cstart cend)
    (let loop ((result ::list '())
               (i ::string-cursor (as string-cursor cend)))
      (if (string-cursor<=? i (as string-cursor start))
@@ -99,21 +91,12 @@
          (let ((prev (string-cursor-prev str i)))
            (loop (make <pair> (string-cursor-ref str prev) result) prev))))))
 
-(define (list->string (lst ::list)) ::string
-  (let* ((len ::int (lst:size))
-	 (result ::abstract-string (make gnu.lists.FString len)))
-    (do ((i ::int 0 (+ i 1)))
-	((>= i len) result)
-      (let ((pair ::pair lst))
-	(string-set! result i pair:car)
-	(set! lst pair:cdr)))))
-
 (define (string-copy (str ::java.lang.CharSequence)
                      #!optional
                      (start ::int 0)
-                     (end ::int -1))
+                     (end ::int 0 supplied-end))
   ::gnu.lists.FString
-  (with-start-end str (start end) (istart iend)
+  (with-start-end str (start end supplied-end) (istart iend)
                   (gnu.lists.FString str istart (- iend istart))))
 
 (define (string-copy! (to ::gnu.lists.FString)
@@ -131,12 +114,12 @@
                          (src ::java.lang.CharSequence)
                          #!optional
                          (sstart ::int 0)
-                         (send ::int -1))
+                         (send ::int 0 supplied-send))
   ::void
   (with-start-end 
-   src (sstart send) (csstart csend)
+   src (sstart send supplied-send) (csstart csend)
    (with-start-end
-    dst (dstart dend) (cdstart cdend)
+    dst (dstart dend #t) (cdstart cdend)
     (dst:replace src csstart csend cdstart cdend))))
 
 (define (string-fill! str ::abstract-string ch ::character
@@ -176,11 +159,6 @@
     (invoke-static <gnu.lists.Strings> 'makeCapitalize copy)
     copy))
 
-(define (string-append #!rest (args :: <Object[]>)) :: <gnu.lists.FString>
-  (let ((str :: <gnu.lists.FString> (make <gnu.lists.FString>)))
-    (invoke str 'addAllStrings args 0)
-    str))
-
 (define (%string-compare-ci2 (str1 :: string) (str2 :: string)) ::int
   ;; The obvious doesn't handle German SS correctly.
   ;;((str1:toString):compareToIgnoreCase (str2:toString)))
@@ -195,7 +173,7 @@
 (define-compare string-ci>=? string >= %string-compare-ci2)
 
 (define (%char-compare (c1 :: character) (c2 :: character)) ::int
-  (let ((i1 (char->integer c1)) (i2 (char->integer c2)))
+  (let ((i1 (as int c1)) (i2 (as int c2)))
     (cond ((> i1 i2) 1) ((< i1 i2) -1) (else 0))))
 
 (define-compare char=? character = %char-compare
@@ -210,8 +188,8 @@
   validate-apply: "kawa.lib.compile_misc:charCompareValidateApply")
 
 (define (%char-compare-ci (c1 :: character) (c2 :: character)) ::int
-  (- (java.lang.Character:toUpperCase (char->integer c1))
-     (java.lang.Character:toUpperCase (char->integer c2))))
+  (- (java.lang.Character:toUpperCase (as int c1))
+     (java.lang.Character:toUpperCase (as int c2))))
 
 (define-compare char-ci=? character = %char-compare-ci
   validate-apply: "kawa.lib.compile_misc:charCompareValidateApply")
@@ -227,26 +205,20 @@
 ;; Hook for backward compatibility with SRFI-13's string-for-each,
 ;; which differs from the R7RS version.
 (define (srfi-13-string-for-each proc str::string #!optional
-                                 (start::int 0) (end::int -1))
+                                 (start::int 0) (end::int 0 end-supplied))
   ::void
-  validate-apply: "kawa.lib.compile_map:stringForEach1ValidateApply" 
-  (let* ((cstart ::string-cursor
-                 (string-cursor-next str
-                                     (as string-cursor 0)
-                                     start))
-         (cend ::string-cursor
-             (if (= end -1) (as string-cursor (str:length))
-                 (string-cursor-next str cstart (- end start)))))
-    (string-cursor-for-each proc str cstart cend)))
-  
+  validate-apply: "kawa.lib.compile_map:stringForEach1ValidateApply"
+  (string-for-each-forwards proc str start end end-supplied))
+
 (define (string-for-each proc str1::string #!rest rst::object[])::void
   validate-apply: "kawa.lib.compile_map:stringForEachValidateApply"
   (define nrst rst:length)
   (cond ((= nrst 0)
          (string-cursor-for-each proc str1))
         ((and (< nrst 3) (not (instance? (rst 0) string)))
-         (srfi-13-string-for-each proc str1 (rst 0)
-                           (if (= nrst 2) (rst 1) -1)))
+         (string-for-each-forwards proc str1 (rst 0)
+                                   (if (= nrst 2) (rst 1) -1)
+                                   (= nrst 2)))
         (else
          (define n::int (+ nrst 1))
          (define cursors::string-cursor[] (string-cursor[] length: n))
@@ -272,35 +244,48 @@
                            (set! (cursors i) (string-cursor-next str curs-i))
                            (loop2 (+ i 1)))))))))))
 
-(define (string-map proc str1::string #!rest rst::string[])::string
-  (define nrst rst:length)
-  (define n::int (+ nrst 1))
-  (define cursors::string-cursor[] (string-cursor[] length: n))
-  (define ends::string-cursor[] (string-cursor[] length: n))
-  (define chs::gnu.text.Char[] (gnu.text.Char[] length: n))
-  (define len1 (str1:length))
-  (define result (gnu.lists.FString:alloc len1))
-  (set! (cursors 0) 0)
-  (set! (ends 0) len1)
-  (do ((i ::int 1 (+ i 1))) ((>= i n))
-    (let ((str ::string (rst (- i 1))))
-      (set! (cursors i) 0)
-      (set! (ends i) (str:length))))
-  (let loop1 ()
-    (let loop2 ((i::int 0))
-      (cond ((= i n)
-             (let ((ch::character (proc @chs)))
-               (result:appendCharacter (as int ch)))
-             (loop1))
-            (else
-             (define curs-i (cursors i))
-             (define end-i (ends i))
-             (define str ::string (if (= i 0) str1 (rst (- i 1))))
-             (cond ((string-cursor<? curs-i end-i)
-                    (set! (chs i) (string-cursor-ref str curs-i))
-                    (set! (cursors i) (string-cursor-next str curs-i))
-                    (loop2 (+ i 1))))))))
-  result)
+(define (string-for-each-index proc str::string #!optional
+                               (start::int 0) (end::int 0 end-supplied))
+  ::void
+  (let* ((cstart ::string-cursor
+                 (string-cursor-next str
+                                     (as string-cursor 0)
+                                     start))
+         (cend ::string-cursor
+             (if (not end-supplied) (as string-cursor (str:length))
+                 (string-cursor-next str cstart (- end start)))))
+    (do ((cursor::string-cursor cstart
+                                (string-cursor-next str cursor))
+         (i ::int start (+ i 1)))
+        ((string-cursor>=? cursor cend))
+      (proc i))))
+
+(define (string-fold kons knil str::string #!optional
+                     (start::int 0) (end::int -1))
+  (let* ((cstart ::string-cursor
+                 (string-cursor-next str
+                                     (as string-cursor 0)
+                                     start))
+         (cend ::string-cursor
+             (if (= end -1) (as string-cursor (str:length))
+                 (string-cursor-next str cstart (- end start))))
+         (result knil))
+    (do ((cursor::string-cursor cstart
+                                (string-cursor-next str cursor)))
+        ((string-cursor>=? cursor cend)
+         result)
+      (set! result (kons (string-cursor-ref str cursor) result)))))
+
+(define (string-fold-right kons knil str::string #!optional
+                     (start::int 0) (end::int 0 supplied-end))
+  (with-start-end 
+   str (start end supplied-end) (cstart cend)
+   (let loop ((result knil)
+              (i ::string-cursor (as string-cursor cend)))
+     (if (string-cursor<=? i (as string-cursor start))
+         result
+         (let ((prev (string-cursor-prev str i)))
+           (loop (kons (string-cursor-ref str prev) result) prev))))))
 
 (define (string-append! str::gnu.lists.FString #!rest args :: object[]) ::void
   validate-apply: "kawa.lib.compile_misc:stringAppendToValidateApply"
@@ -309,3 +294,120 @@
       ((>= i len))
       (str:append (args i)))))
 
+(define (xsubstring str::string
+                    #!optional (from::int 0) (to::int 0 supplied-to)
+                    (start::int 0) (end::int 0 supplied-end))
+  ::string
+  (let* ((end (if supplied-end end (gnu.lists.Strings:sizeInCodePoints str))))
+    (gnu.lists.IString
+     (gnu.lists.Strings:replicate from to supplied-to str start end supplied-end))))
+
+(define (string->utf8
+         (v ::string)
+         #!optional
+         (start ::int 0)
+         (end ::int 0 supplied-end))
+  ::bytevector
+  (with-start-end
+   v (start end supplied-end) (cstart cend)
+   (gnu.lists.U8Vector
+    (((v:toString):substring cstart cend):getBytes
+     (cond-expand (java-7 java.nio.charset.StandardCharsets:UTF_8)
+                  (else "UTF-8"))))))
+
+(define (string->utf16
+         (v ::string)
+         #!optional
+         (start ::int 0)
+         (end ::int 0 supplied-end))
+  ::bytevector  (with-start-end
+   v (start end supplied-end) (cstart cend)
+   (gnu.lists.U8Vector
+    (gnu.lists.Strings:toUtf16 v cstart cend
+                               (cond-expand (big-endian #t) (else #f))
+                               #t))))
+
+(define (string->utf16be
+         (v ::string)
+         #!optional
+         (start ::int 0)
+         (end ::int 0 supplied-end))
+  ::bytevector  (with-start-end
+   v (start end supplied-end) (cstart cend)
+   (gnu.lists.U8Vector
+    (gnu.lists.Strings:toUtf16 v cstart cend #t #f))))
+
+(define (string->utf16le
+         (v ::string)
+         #!optional
+         (start ::int 0)
+         (end ::int 0 supplied-end))
+  ::bytevector  (with-start-end
+   v (start end supplied-end) (cstart cend)
+   (gnu.lists.U8Vector
+    (gnu.lists.Strings:toUtf16 v cstart cend #f #f))))
+
+(define (string-tabulate proc (len ::int))
+  (let ((result (gnu.lists.FString:alloc len)))
+    (do ((i ::int 0 (+ i 1)))
+        ((>= i len)
+         (gnu.lists.IString result))
+      (result:append (proc i)))))
+
+(define (string-unfold p f g seed #!optional (base "") (make-final #!null))
+  (let ((result (gnu.lists.FString)))
+    (result:append base)
+    (let recur ((seed seed))
+      (cond ((p seed)
+             (if (not (eq? make-final #!null))
+                 (result:append (make-final seed))))
+            (else
+             (result:append (f seed))
+             (recur (g seed)))))
+    (gnu.lists.IString result)))
+
+(define (string-unfold-right p f g seed #!optional (base "") (make-final #!null))
+  (let ((result (gnu.lists.FString)))
+    (result:prepend base)
+    (let recur ((seed seed))
+      (cond ((p seed)
+             (if (not (eq? make-final #!null))
+                 (result:prepend (make-final seed))))
+            (else
+             (result:prepend (f seed))
+             (recur (g seed)))))
+    (gnu.lists.IString result)))
+
+(define (string-pad (str ::java.lang.CharSequence)
+                    len::int
+                    #!optional
+                    (ch ::character #\Space)
+                    (start ::int 0)
+                    (end ::int (gnu.lists.Strings:sizeInCodePoints str)))
+  (let ((olen (- end start)))
+    (if (<= len olen)
+        (gnu.lists.IString:valueOf str (- end len) len)
+        (let ((result (gnu.lists.FString:alloc len)))
+          (do ((i ::int (- len olen) (- i 1)))
+              ((<= i 0))
+            (result:appendCharacter (as int ch)))
+          (with-start-end str (start end #t) (cstart cend)
+                          (result:append str cstart cend))
+          (gnu.lists.IString result)))))
+
+(define (string-pad-right (str ::java.lang.CharSequence)
+                          len::int
+                          #!optional
+                          (ch ::character #\Space)
+                          (start ::int 0)
+                          (end ::int (gnu.lists.Strings:sizeInCodePoints str)))
+  (let ((olen (- end start)))
+    (if (<= len olen)
+        (gnu.lists.IString:valueOf str start len)
+        (let ((result (gnu.lists.FString:alloc len)))
+          (with-start-end str (start end #t) (cstart cend)
+                          (result:append str cstart cend))
+          (do ((i ::int (- len olen) (- i 1)))
+              ((<= i 0))
+            (result:appendCharacter (as int ch)))
+          result))))
