@@ -1,13 +1,6 @@
 package kawa.standard;
-import kawa.lang.*;
-import gnu.mapping.*;
 
-public class char_ready_p extends Procedure0or1 {
-  public final Object apply0 ()
-  {
-    return apply1 (InPort.inDefault ());
-  }
-  
+public class char_ready_p {
   public static boolean ready (Object arg1)
   {
     try
@@ -24,9 +17,4 @@ public class char_ready_p extends Procedure0or1 {
 	return false;
       }
   }
-
-  public final Object apply1 (Object arg1)
-  {
-    return Scheme.boolObject(ready(arg1));
-  }
 }
