@@ -1,5 +1,5 @@
 ;; -*- coding: utf-8 -*-
-(test-begin "arrays" 228)
+(test-begin "arrays" 234)
 
 ;;; array test
 ;;; 2001 Jussi Piitulainen
@@ -222,6 +222,7 @@
 	(art (share-array (make-array shp) shp (lambda (k) k))))
     (array-set! shp 0 0 '?)
     (array-set! shp 0 1 '!)
+    (test-equal #2s32((10 12)) (array-shape arr))
     (test-equal 2 (array-rank shp))
     (test-equal 0 (array-start shp 0))
     (test-equal 1 (array-end shp 0))
@@ -475,6 +476,25 @@
                            (let ((x (ind 0)) (y (ind 1)))
                              (+ 10 x (- y))))))
 
+(define (make-sparse-array shape default-value)
+  (let ((vals '()))
+    (build-array shape
+                 (lambda (I)
+                   (let ((v (assoc I vals)))
+                     (if v (cdr v)
+                         default-value)))
+                 (lambda (I newval)
+                   (let ((v (assoc I vals)))
+                     (if v
+                         (set-cdr! v newval)
+                         (set! vals (cons (cons I newval) vals))))))))
+
+(define sarr (make-sparse-array [3 4] -1))
+(array-set! sarr 1 1 10)
+(array-set! sarr 2 3 23)
+(array-set! sarr 1 1 11)
+(test-equal #2a((-1 -1 -1 -1) (-1 11 -1 -1) (-1 -1 -1 23)) sarr)
+
 (test-equal &{&-
 #2a@10:2:3
 ║10│ 9│8║
@@ -488,15 +508,15 @@
                     (- x y))))))
 
 (test-equal &{&-
-╔#2a:2:3╗
+#2a:2:3═╗
 ║12│3│ 4║
 ╟──┼─┼──╢
 ║ 5│9│11║
 ╚══╧═╧══╝}
-  (format-array #2a((12 3 4) (5 9 11))))
+  (format-array #2a((12 3 4) (5 9 11)) #f))
 
 (test-equal &{&-
-╔#2a:2:4╤═══╗
+#2a:2:4═╤═══╗
 ║ab│c│d │e  ║
 ╟──┼─┼──┼───╢
 ║f │g│hi│jkl║
@@ -504,7 +524,7 @@
   (format-array #2a((ab c d e) (f g hi jkl))))
 
 (test-equal &{&-
-╔#2s8:2:3═══╗
+#2s8:2:3╤═══╗
 ║012│003│004║
 ╟───┼───┼───╢
 ║005│-09│011║
@@ -512,7 +532,7 @@
   (format-array #2S8((12 3 4) (5 -9 11)) "~3,'0d"))
 
 (test-equal &{&-
-╔#3a:2:2:3╗
+#3a:2:2:3═╗
 ║ab│c  │d ║
 ╟──┼───┼──╢
 ║e │f  │gh║
@@ -524,7 +544,7 @@
   (format-array #3a(((ab c d) (e f gh)) ((i j k) (lm nop q)))))
 
 (test-equal &{&-
-╔#2a:2:3═══╤═════════╗
+#2a:2:3════╤═════════╗
 ║  334│4545│#2f32:1:2║
 ║     │    │║5.0│6.0║║
 ║     │    │╚═══╧═══╝║
@@ -538,19 +558,26 @@
   (format-array
    #2a((334 4545 #2f32((5 6))) (78987 "abc\ndefg\nhi" #2A((1 2) (3 14))))))
 
-(test-equal "#0a -02" (format-array #0a -2 "~3,'0d"))
+(test-equal "#0a -02" (format-array #0a -2 #f "~3,'0d"))
 
 (test-equal "#2a@1:3:0 ()"
             (format-array #2a@1:3:0()))
 
-(test-equal "#2a@1:2:3((a -9 c) (d 153 ef))"
-            (format "~a" #2a@1:2:3((a -9 "c") (d 153 "ef"))))
+(let ((arr #2a@1:2:3((a -9 "c") (d 153 "ef"))))
+  (test-equal "#2a@1:2:3((a -9 c) (d 153 ef))"
+              (format "~a" arr))
+  (test-equal #2s32((1 3) (0 3)) (array-shape arr)))
+
+(test-equal #2s32((0 3) (2 5) (0 9) (-1 3))
+            (->shape `#(3 (2 5) 9 ,[-1 size: 4])))
 
 (test-equal &{#2a@1:2:3((a -9 "c") (d 153 "ef"))}
             (format "~w" #2a@1:2:3((a -9 "c") (d 153 "ef"))))
 
 (test-equal [2 5] [2 by: 3 <: 8])
 (test-equal [2 5 8] [2 by: 3 <=: 8])
+(test-equal [1 4 7 10] [1 by: 3 size: 4])
+(test-equal [3 4 5 6] [3 size: 4])
 (test-equal [2 5 8] [2 by: 3 <: 9])
 (test-equal [2 5 8 11] [2 by: 3 <=: 11])
 (test-equal [2 5 8 11] [2 by: 3 <=: 13])
